@@ -29,7 +29,7 @@ class Classifier:
     also optimize the engine and output visualizations.
 
     Attributes:
-        model (object): The machine learning model that is created
+        model: The machine learning model that is created
         
         load: load model
 
@@ -37,6 +37,8 @@ class Classifier:
 
         predict: predict new samples
 
+        plot_hyper_opt: Plot hyperparameter optimization history
+
     Args:
         blob_data (ndarray, optional): 2D array of size (n x m), where n is the
             number of samples, and m the number of features. Defaults to None,
@@ -49,6 +51,8 @@ class Classifier:
         optimize (bool): If True the Boruta algorithm will be run to identify the features
             that contain useful information, after which the optimal Random Forest hyperparameters
             will be calculated using Bayesian optimization. 
+        limit_search (bool): If True the CNN optimization will search optimize only three hyperparameters,
+            the batch size, learning rate, and loss function. Defaults to True.
         impute (bool): If False no data imputation will be performed. Defaults to True,
             which will result in two outputs, the classifier and the imputer to save
             for future transformations. 
@@ -57,22 +61,18 @@ class Classifier:
             algorithm. Defaults to 'KNN'.
         n_iter (int): The maximum number of iterations to perform during 
             the hyperparameter search. Defaults to 25. 
-        boruta_trials (int): The number of trials to run when running Boruta for
-            feature selection. Set to 0 for no feature selection. Defaults to 50.
-        balance (bool, optional): If True, a weights array will be calculated and used
-            when fitting the classifier. This can improve classification when classes
-            are imbalanced. This is only applied if the classification is a binary task. 
-            Defaults to True.        
-        
+
     Returns:
         Trained machine learning model.
 
     """
-    def __init__(self, blob_data=None, other_data=None, optimize=True, metric='val_loss', n_iter=25, img_num_channels=1, normalize=True, min_pixel=638,
-        max_pixel=3000, val_X=None, val_Y=None, epochs=100, train_epochs=25):
+    def __init__(self, blob_data=None, other_data=None, optimize=True, limit_search=True, metric='loss', n_iter=25, img_num_channels=1, normalize=True, min_pixel=638,
+        max_pixel=3000, val_X=None, val_Y=None, epochs=100, train_epochs=25, patience=5):
+
         self.blob_data = blob_data
         self.other_data = other_data
         self.optimize = optimize 
+        self.limit_search = limit_search
         self.metric = metric 
         self.n_iter = n_iter
 
@@ -84,6 +84,7 @@ class Classifier:
         self.val_Y = val_Y
         self.epochs = epochs
         self.train_epochs = train_epochs
+        self.patience = patience
 
         self.model = None
         self.history = None 
@@ -105,19 +106,24 @@ class Classifier:
                 min_pixel=self.min_pixel, max_pixel=self.max_pixel, val_X=self.val_X, val_Y=self.val_Y, epochs=self.epochs)
             return      
 
-        self.best_params, self.optimization_results = hyper_opt(self.blob_data, self.other_data, clf='cnn', metric='val_loss', n_iter=self.n_iter, 
+        self.best_params, self.optimization_results = hyper_opt(self.blob_data, self.other_data, clf='cnn', metric=self.metric, n_iter=self.n_iter, 
             balance=False, return_study=True, img_num_channels=self.img_num_channels, normalize=self.normalize, min_pixel=self.min_pixel, 
-            max_pixel=self.max_pixel, val_X=self.val_X, val_Y=self.val_Y, train_epochs=self.train_epochs)
+            max_pixel=self.max_pixel, val_X=self.val_X, val_Y=self.val_Y, train_epochs=self.train_epochs, patience=self.patience, limit_search=self.limit_search)
 
         print("Fitting and returning final model...")
-        self.model, self.history = pyBIA_model(self.blob_data, self.other_data, img_num_channels=self.img_num_channels, normalize=self.normalize,
-            min_pixel=self.min_pixel, max_pixel=self.max_pixel, val_X=self.val_X, val_Y=self.val_Y, epochs=self.epochs, batch_size=self.best_params['batch_size'],
-            lr=self.best_params['lr'], batch_norm=self.best_params['batch_norm'], momentum=self.best_params['momentum'],
-            decay=self.best_params['decay'], nesterov=self.best_params['nesterov'], loss=self.best_params['loss'],
-            padding=self.best_params['padding'], dropout_1=self.best_params['dropout_1'], dropout_2=self.best_params['dropout_2'],
-            activation_conv=self.best_params['activation_conv'], activation_dense=self.best_params['activation_dense'],
-            maxpool_size=self.best_params['maxpool_size'], maxpool_stride=self.best_params['maxpool_stride']) #params
-        
+        if limit_search:
+            self.model, self.history = pyBIA_model(self.blob_data, self.other_data, img_num_channels=self.img_num_channels, normalize=self.normalize,
+                min_pixel=self.min_pixel, max_pixel=self.max_pixel, val_X=self.val_X, val_Y=self.val_Y, epochs=self.epochs, batch_size=self.best_params['batch_size'],
+                lr=self.best_params['lr'], loss=self.best_params['loss'])
+        else:
+            self.model, self.history = pyBIA_model(self.blob_data, self.other_data, img_num_channels=self.img_num_channels, normalize=self.normalize,
+                min_pixel=self.min_pixel, max_pixel=self.max_pixel, val_X=self.val_X, val_Y=self.val_Y, epochs=self.epochs, batch_size=self.best_params['batch_size'],
+                lr=self.best_params['lr'], batch_norm=self.best_params['batch_norm'], momentum=self.best_params['momentum'],
+                decay=self.best_params['decay'], nesterov=self.best_params['nesterov'], loss=self.best_params['loss'],
+                padding=self.best_params['padding'], dropout_1=self.best_params['dropout_1'], dropout_2=self.best_params['dropout_2'],
+                activation_conv=self.best_params['activation_conv'], activation_dense=self.best_params['activation_dense'],
+                maxpool_size=self.best_params['maxpool_size'], maxpool_stride=self.best_params['maxpool_stride']) #params
+            
         return 
 
     def save(self, path=None, overwrite=False):
@@ -157,13 +163,13 @@ class Classifier:
         
         path += 'pyBIA_cnn_model/'
         if self.model is not None:      
-            np.savetxt(path+'model_acc', history.history['accuracy'])
-            np.savetxt(path+'model_loss', history.history['loss'])
+            np.savetxt(path+'model_acc', self.history.history['accuracy'])
+            np.savetxt(path+'model_loss', self.history.history['loss'])
             if self.val_X is not None:
-                np.savetxt(path+'model_val_acc', history.history['val_accuracy'])
-                np.savetxt(path+'model_val_loss', history.history['val_loss'])
+                np.savetxt(path+'model_val_acc', self.history.history['val_accuracy'])
+                np.savetxt(path+'model_val_loss', self.history.history['val_loss'])
 
-            save_model(model, path+'Keras_Model.h5')
+            save_model(self.model, path+'Keras_Model.h5')
 
         if self.best_params is not None:
             joblib.dump(self.best_params, path+'Best_Params')
